@@ -30,24 +30,21 @@ from gettext import gettext as _
 from logging import getLogger
 from shutil import copyfileobj
 import io
-import json
 import operator
 import os
 import re
 import sys
 
-from plainbox.impl.applogic import get_whitelist_by_name
+from plainbox.abc import IJobResult
 from plainbox.impl.commands.inv_run import RunInvocation
 from plainbox.impl.exporter import ByteStringStreamTranslator
-from plainbox.impl.exporter import get_all_exporters
-from plainbox.impl.exporter.html import HTMLSessionStateExporter
-from plainbox.impl.exporter.xml import XMLSessionStateExporter
 from plainbox.impl.secure.config import Unset, ValidationError
+from plainbox.impl.secure.origin import CommandLineTextSource
 from plainbox.impl.secure.origin import Origin
 from plainbox.impl.secure.qualifiers import FieldQualifier
 from plainbox.impl.secure.qualifiers import OperatorMatcher
+from plainbox.impl.secure.qualifiers import RegExpJobQualifier
 from plainbox.impl.secure.qualifiers import select_jobs
-from plainbox.impl.secure.qualifiers import WhiteList
 from plainbox.impl.session import SessionMetaData
 from plainbox.impl.transport import get_all_transports
 from plainbox.impl.transport import TransportError
@@ -56,6 +53,7 @@ from plainbox.vendor.textland import get_display
 from checkbox_ng.misc import SelectableJobTreeNode
 from checkbox_ng.ui import ScrollableTreeNode
 from checkbox_ng.ui import ShowMenu
+from checkbox_ng.ui import ShowRerun
 from checkbox_ng.ui import ShowWelcome
 
 
@@ -72,8 +70,8 @@ class CliInvocation2(RunInvocation):
         launcher specific to 'checkbox cli'
     :ivar _display:
         A textland display object
-    :ivar _whitelists:
-        A list of whitelists to look at
+    :ivar _qualifier_list:
+        A list of job qualifiers used to build the session desired_job_list
     """
 
     def __init__(self, provider_loader, config_loader, ns, launcher,
@@ -83,8 +81,9 @@ class CliInvocation2(RunInvocation):
             display = get_display()
         self._launcher = launcher
         self._display = display
-        self._whitelists = []
-        self.select_whitelist()
+        self._qualifier_list = []
+        self._testplan_list = []
+        self.select_qualifier_list()
 
     @property
     def launcher(self):
@@ -100,15 +99,51 @@ class CliInvocation2(RunInvocation):
         """
         return self._display
 
-    def select_whitelist(self):
+    def select_qualifier_list(self):
+        # Add whitelists
         if 'whitelist' in self.ns and self.ns.whitelist:
-            for whitelist in self.ns.whitelist:
-                self._whitelists.append(WhiteList.from_file(whitelist.name))
-        elif self.config.whitelist is not Unset:
-            self._whitelists.append(WhiteList.from_file(self.config.whitelist))
-        elif ('include_pattern_list' in self.ns and
-              self.ns.include_pattern_list):
-            self._whitelists.append(WhiteList(self.ns.include_pattern_list))
+            for whitelist_file in self.ns.whitelist:
+                qualifier = self.get_whitelist_from_file(
+                    whitelist_file.name, whitelist_file)
+                if qualifier is not None:
+                    self._qualifier_list.append(qualifier)
+        # Add all the --include jobs
+        for pattern in self.ns.include_pattern_list:
+            origin = Origin(CommandLineTextSource('-i', pattern), None, None)
+            try:
+                qualifier = RegExpJobQualifier(
+                    '^{}$'.format(pattern), origin, inclusive=True)
+            except Exception as exc:
+                logger.warning(
+                    _("Incorrect pattern %r: %s"), pattern, exc)
+            else:
+                self._qualifier_list.append(qualifier)
+        # Add all the --exclude jobs
+        for pattern in self.ns.exclude_pattern_list:
+            origin = Origin(CommandLineTextSource('-x', pattern), None, None)
+            try:
+                qualifier = RegExpJobQualifier(
+                    '^{}$'.format(pattern), origin, inclusive=False)
+            except Exception as exc:
+                logger.warning(
+                    _("Incorrect pattern %r: %s"), pattern, exc)
+            else:
+                self._qualifier_list.append(qualifier)
+        if self.config.whitelist is not Unset:
+            self._qualifier_list.append(
+                self.get_whitelist_from_file(self.config.whitelist))
+
+    def select_testplan(self):
+        # Add the test plan
+        if self.ns.test_plan is not None:
+            for provider in self.provider_list:
+                for unit in provider.id_map[self.ns.test_plan]:
+                    if unit.Meta.name == 'test plan':
+                        self._qualifier_list.append(unit.get_qualifier())
+                        self._testplan_list.append(unit)
+                        return
+            else:
+                logger.error(_("There is no test plan: %s"), self.ns.test_plan)
 
     def run(self):
         return self.do_normal_sequence()
@@ -125,9 +160,8 @@ class CliInvocation2(RunInvocation):
 
             For now just look for changes as compared to run.py's version.
         """
-        # Create exporter and transport early so that we can handle bugs
-        # before starting the session.
-        self.create_exporter()
+        # Create transport early so that we can handle bugs before starting the
+        # session.
         self.create_transport()
         if self.is_interactive:
             resumed = self.maybe_resume_session()
@@ -142,8 +176,13 @@ class CliInvocation2(RunInvocation):
         if not resumed:
             # Show the welcome message
             self.show_welcome_screen()
+            # Process testplan command line options
+            self.select_testplan()
             # Maybe allow the user to do a manual whitelist selection
-            self.maybe_interactively_select_whitelists()
+            if not self._qualifier_list:
+                self.maybe_interactively_select_testplans()
+            if self._testplan_list:
+                self.manager.test_plans = tuple(self._testplan_list)
             # Store the application-identifying meta-data and checkpoint the
             # session.
             self.store_application_metadata()
@@ -151,11 +190,10 @@ class CliInvocation2(RunInvocation):
             self.manager.checkpoint()
             # Run all the local jobs. We need to do this to see all the things
             # the user may select
-            self.select_local_jobs()
-            self.run_all_selected_jobs()
+            if self.is_interactive:
+                self.select_local_jobs()
+                self.run_all_selected_jobs()
             self.interactively_pick_jobs_to_run()
-        else:
-            self.load_app_blob()
         # Maybe ask the secure launcher to prompt for the password now. This is
         # imperfect as we are going to run local jobs and we cannot see if they
         # might need root or not. This cannot be fixed before template jobs are
@@ -164,6 +202,8 @@ class CliInvocation2(RunInvocation):
         self.maybe_warm_up_authentication()
         self.print_estimated_duration()
         self.run_all_selected_jobs()
+        if self.is_interactive:
+            self.maybe_rerun_jobs()
         self.export_and_send_results()
         if SessionMetaData.FLAG_INCOMPLETE in self.metadata.flags:
             print(self.C.header("Session Complete!", "GREEN"))
@@ -173,78 +213,61 @@ class CliInvocation2(RunInvocation):
 
     def store_application_metadata(self):
         super().store_application_metadata()
-        self.metadata.app_blob = json.dumps([
-            whitelist.origin.source.filename
-            for whitelist in self._whitelists
-        ]).encode("UTF-8")
-        logger.info("Saved whitelist mask: %r", self._whitelists)
-
-    def load_app_blob(self):
-        whitelist_filename_list = json.loads(
-            self.metadata.app_blob.decode("UTF-8"))
-        whitelist_list = [
-            WhiteList.from_file(filename)
-            for filename in whitelist_filename_list]
-        self._whitelists = whitelist_list
-        logger.info("Loaded whitelist mask: %r", self._whitelists)
+        self.metadata.app_blob = b''
 
     def show_welcome_screen(self):
         text = self.launcher.text
         if self.is_interactive and text:
             self.display.run(ShowWelcome(text))
 
-    def maybe_interactively_select_whitelists(self):
+    def maybe_interactively_select_testplans(self):
         if self.launcher.skip_whitelist_selection:
-            self._whitelists.extend(self.get_default_whitelists())
-        elif self.is_interactive and not self._whitelists:
-            self._whitelists.extend(self.get_interactively_picked_whitelists())
-        else:
-            self._whitelists.extend(self.get_default_whitelists())
-        logger.info(_("Selected whitelists: %r"), self._whitelists)
+            self._qualifier_list.extend(self.get_default_testplans())
+        elif self.is_interactive:
+            self._qualifier_list.extend(
+                self.get_interactively_picked_testplans())
+        elif self.launcher.whitelist_selection:
+            self._qualifier_list.extend(self.get_default_testplans())
+        logger.info(_("Selected testplans: %r"), self._qualifier_list)
 
-    def get_interactively_picked_whitelists(self):
+    def get_interactively_picked_testplans(self):
         """
         Show an interactive dialog that allows the user to pick a list of
-        whitelists. The set of whitelists is limited to those offered by the
+        testplans. The set of testplans is limited to those offered by the
         'default_providers' setting.
 
         :returns:
-            A list of selected whitelists
+            A list of selected testplans
         """
-        whitelist_name_list = whitelist_selection = []
+        testplans = []
+        testplan_selection = []
         for provider in self.provider_list:
-            whitelist_name_list.extend([
-                whitelist.name for whitelist in
-                provider.get_builtin_whitelists() if re.search(
-                    self.launcher.whitelist_filter, whitelist.name)])
-        whitelist_selection = [
-            whitelist_name_list.index(w) for w in whitelist_name_list if
-            re.search(self.launcher.whitelist_selection, w)]
+            testplans.extend(
+                [unit for unit in provider.unit_list if
+                 unit.Meta.name == 'test plan' and
+                 re.search(self.launcher.whitelist_filter, unit.partial_id)])
+        testplan_name_list = [testplan.tr_name() for testplan in testplans]
+        testplan_selection = [
+            testplans.index(t) for t in testplans if
+            re.search(self.launcher.whitelist_selection, t.partial_id)]
         selected_list = self.display.run(
-            ShowMenu(_("Suite selection"), whitelist_name_list,
-                     whitelist_selection))
+            ShowMenu(_("Suite selection"), testplan_name_list,
+                     testplan_selection))
         if not selected_list:
-            raise SystemExit(_("No whitelists selected, aborting"))
-        return [get_whitelist_by_name(
-            self.provider_list, whitelist_name_list[selected_index])
-            for selected_index in selected_list]
+            raise SystemExit(_("No testplan selected, aborting"))
+        self._testplan_list.extend(
+            [testplans[selected_index] for selected_index in selected_list])
+        return [testplans[selected_index].get_qualifier() for selected_index
+                in selected_list]
 
-    def get_default_whitelists(self):
-        whitelist_name_list = []
+    def get_default_testplans(self):
+        testplans = []
         for provider in self.provider_list:
-            whitelist_name_list.extend([
-                w for w in provider.get_builtin_whitelists() if re.search(
-                    self.launcher.whitelist_selection, w.name)])
-        return whitelist_name_list
-
-    def create_exporter(self):
-        """
-        Create the ISessionStateExporter based on the command line options
-
-        This sets the :ivar:`_exporter`.
-        """
-        # TODO:
-        self._exporter = None
+            testplans.extend([
+                unit.get_qualifier() for unit in provider.unit_list if
+                unit.Meta.name == 'test plan' and re.search(
+                    self.launcher.whitelist_selection, unit.partial_id)])
+        return testplans
 
     def create_transport(self):
         """
@@ -269,7 +292,7 @@ class CliInvocation2(RunInvocation):
         # within, we only need to and an exclusive qualifier that deselects
         # non-local jobs and we're done.
         qualifier_list = []
-        qualifier_list.extend(self._whitelists)
+        qualifier_list.extend(self._qualifier_list)
         origin = Origin.get_caller_origin()
         qualifier_list.append(FieldQualifier(
             'plugin', OperatorMatcher(operator.ne, 'local'), origin,
@@ -281,12 +304,11 @@ class CliInvocation2(RunInvocation):
     def interactively_pick_jobs_to_run(self):
         print(self.C.header(_("Selecting Jobs For Execution")))
         self._update_desired_job_list(select_jobs(
-            self.manager.state.job_list, self._whitelists))
-        if self.launcher.skip_test_selection:
+            self.manager.state.job_list, self._qualifier_list))
+        if self.launcher.skip_test_selection or not self.is_interactive:
             return
         tree = SelectableJobTreeNode.create_tree(
-            self.manager.state.run_list,
-            legacy_mode=True)
+            self.manager.state, self.manager.state.run_list)
         title = _('Choose tests to run on your system:')
         self.display.run(ScrollableTreeNode(tree, title))
         # NOTE: tree.selection is correct but ordered badly.  To retain
@@ -300,10 +322,10 @@ class CliInvocation2(RunInvocation):
     def export_and_send_results(self):
         if self.is_interactive:
             print(self.C.header(_("Results")))
-            exporter = get_all_exporters()['text']()
+            exporter = self.manager.create_exporter(
+                '2013.com.canonical.plainbox::text')
             exported_stream = io.BytesIO()
-            data_subset = exporter.get_session_data_subset(self.manager.state)
-            exporter.dump(data_subset, exported_stream)
+            exporter.dump_from_session_manager(self.manager, exported_stream)
             exported_stream.seek(0)  # Need to rewind the file, puagh
             # This requires a bit more finesse, as exporters output bytes
             # and stdout needs a string.
@@ -317,39 +339,30 @@ class CliInvocation2(RunInvocation):
             "plainbox")
         if not os.path.exists(base_dir):
             os.makedirs(base_dir)
-        results_file = os.path.join(base_dir, 'results.html')
-        submission_file = os.path.join(base_dir, 'submission.xml')
-        exporter_list = [XMLSessionStateExporter, HTMLSessionStateExporter]
-        if 'xlsx' in get_all_exporters():
-            from plainbox.impl.exporter.xlsx import XLSXSessionStateExporter
-            exporter_list.append(XLSXSessionStateExporter)
-        # We'd like these options for our reports.
         exp_options = ['with-sys-info', 'with-summary', 'with-job-description',
-                       'with-text-attachments']
-        for exporter_cls in exporter_list:
-            # Exporters may support different sets of options, ensure we don't
-            # pass an unsupported one (which would cause a crash)
-            actual_options = [opt for opt in exp_options
-                              if opt in exporter_cls.supported_option_list]
-            exporter = exporter_cls(actual_options)
-            data_subset = exporter.get_session_data_subset(self.manager.state)
-            results_path = results_file
-            if exporter_cls is XMLSessionStateExporter:
-                results_path = submission_file
-            # FIXME: replacing extension is ugly
-            if 'xlsx' in get_all_exporters():
-                if exporter_cls is XLSXSessionStateExporter:
-                    results_path = results_path.replace('html', 'xlsx')
-            with open(results_path, "wb") as stream:
-                exporter.dump(data_subset, stream)
+                       'with-text-attachments', 'with-certification-status',
+                       'with-job-defs', 'with-io-log', 'with-comments']
         print()
-        print(_("Saving submission file to {}").format(submission_file))
-        self.submission_file = submission_file
-        print(_("View results") + " (HTML): file://{}".format(results_file))
-        if 'xlsx' in get_all_exporters():
-            # FIXME: replacing extension is ugly
-            print(_("View results") + " (XLSX): file://{}".format(
-                results_file.replace('html', 'xlsx')))
+        if self.launcher.exporter is not Unset:
+            exporters = self.launcher.exporter
+        else:
+            exporters = [
+                '2013.com.canonical.plainbox::hexr',
+                '2013.com.canonical.plainbox::html',
+                '2013.com.canonical.plainbox::xlsx',
+                '2013.com.canonical.plainbox::json',
+            ]
+        for unit_name in exporters:
+            exporter = self.manager.create_exporter(
+                unit_name, exp_options, strict=False)
+            extension = exporter.unit.file_extension
+            results_path = os.path.join(
+                base_dir, 'submission.{}'.format(extension))
+            with open(results_path, "wb") as stream:
+                exporter.dump_from_session_manager(self.manager, stream)
+            print(_("View results") + " ({}): file://{}".format(
+                extension, results_path))
+        self.submission_file = os.path.join(base_dir, 'submission.xml')
         if self.launcher.submit_to is not Unset:
             if self.launcher.submit_to == 'certification':
                 # If we supplied a submit_url in the launcher, it
@@ -365,6 +378,10 @@ class CliInvocation2(RunInvocation):
                 # override launcher (which is not)
                 if not self.config.secure_id:
                     self.config.secure_id = self.launcher.secure_id
+                # Override the secure_id configuration with the one provided
+                # by the command-line option
+                if self.ns.secure_id:
+                    self.config.secure_id = self.ns.secure_id
                 if self.config.secure_id is Unset:
                     again = True
                     if not self.is_interactive:
@@ -468,3 +485,31 @@ class CliInvocation2(RunInvocation):
                             ": {0}").format(result))
             except TransportError as exc:
                 print(str(exc))
+
+    def maybe_rerun_jobs(self):
+        def rerun_predicate(job_state):
+            return job_state.result.outcome in (
+                IJobResult.OUTCOME_FAIL, IJobResult.OUTCOME_CRASH)
+        # create a list of jobs that qualify for rerunning
+        rerun_candidates = []
+        for job in self.manager.state.run_list:
+            if rerun_predicate(self.manager.state.job_state_map[job.id]):
+                rerun_candidates.append(job)
+        # bail-out early if no job qualifies for rerunning
+        if not rerun_candidates:
+            return
+        tree = SelectableJobTreeNode.create_tree(
+            self.manager.state, rerun_candidates)
+        # deselect all by default
+        tree.set_descendants_state(False)
+        self.display.run(ShowRerun(tree, _("Select jobs to re-run")))
+        wanted_set = frozenset(tree.selection)
+        if not wanted_set:
+            # nothing selected - nothing to run
+            return
+        # reset outcome of jobs that are selected for re-running
+        for job in wanted_set:
+            from plainbox.impl.result import MemoryJobResult
+            self.manager.state.job_state_map[job.id].result = \
+                MemoryJobResult({})
+        self.run_all_selected_jobs()
