@@ -79,7 +79,8 @@ class ServiceInvocation:
         logger.info(_("Setting up DBus objects..."))
         session_list = []  # TODO: load sessions
         logger.debug(_("Constructing Service object"))
-        service_obj = Service(self.provider_loader(), session_list, self.config_loader())
+        service_obj = Service(
+            self.provider_loader(), session_list, self.config_loader())
         logger.debug(_("Constructing ServiceWrapper"))
         service_wrp = ServiceWrapper(service_obj, on_exit=lambda: loop.quit())
         logger.info(_("Publishing all objects on DBus"))
@@ -116,7 +117,8 @@ class ServiceCommand(CheckboxCommand):
     """
 
     def invoked(self, ns):
-        return ServiceInvocation(self.provider_loader, self.config_loader, ns).run()
+        return ServiceInvocation(
+            self.provider_loader, self.config_loader, ns).run()
 
     def register_parser(self, subparsers):
         parser = subparsers.add_parser("service", help=_("spawn dbus service"))
